"use strict";

var AbstractMarker = require('./AbstractMarker');

/**
 * This class describes Google maps marker (connected to {@link Alias}) that is
 * visualized on the map.
 */
function AliasMarker(params) {
  AbstractMarker.call(this, params);
}
AliasMarker.prototype = Object.create(AbstractMarker.prototype);
AliasMarker.prototype.constructor = AliasMarker;

/**
 * Returns {@link Alias} data for this marker.
 * 
 * @returns {@link Alias} data for this marker
 */
AliasMarker.prototype.getAliasData = function() {
  return this._aliasData;
};

/**
 * Sets {@link Alias} data for this marker.
 * 
 * @param data
 *          {@link Alias} data for this marker
 */
AliasMarker.prototype.setAliasData = function(data) {
  this._aliasData = data;
};

/**
 * Returns coordinates where marker is pointing.
 * 
 * @returns {google.maps.Point} - coordinates where marker is pointing
 */
AliasMarker.prototype.getCoordinates = function() {
  return new google.maps.Point(this._aliasData.x + this._aliasData.width / 2, this._aliasData.y
      + this._aliasData.height / 2);
};

AliasMarker.prototype.init = function() {
  var self = this;
  var model = self.getCustomMap().getModel();
  return model.getByIdentifiedElement(self.getIdentifiedElement()).then(function(aliasData) {
    self.setAliasData(aliasData);
    return self._init();
  }).then(function() {
    self.show();
  });
};

module.exports = AliasMarker;
